/*
 * Synopsis:
 *	signed char disrsc(int stream, int *retval)
 *
 *	Gets a Data-is-Strings signed integer from <stream>, converts it into a
 *	signed char, and returns it.  The signed integer in <stream> consists
 *	of a counted string of digits, starting with a zero or a minus sign,
 *	which represents the number.  If the number doesn't lie between -9 and
 *	9, inclusive, it is preceeded by at least one count.
 *
 *	This format for character strings representing signed integers can best
 *	be understood through the decoding algorithm:
 *
 *	1. Initialize the digit count to 1.
 *
 *	2. Read the next digit; if it is a sign, go to step (4).
 *
 *	3. Decode a new count from the digit decoded in step (2) and the next
 *	   count - 1 digits; repeat step (2).
 *
 *	4. Decode the next count digits as the magnitude of the signed integer.
 *
 *	*<retval> gets DIS_SUCCESS if everything works well.  It gets an error
 *	code otherwise.  In case of an error, the <stream> character pointer is
 *	reset, making it possible to retry with some other conversion strategy.
 */

#include <pbs_config.h>   /* the master config generated by configure */

#include <assert.h>
#include <stddef.h>

#include "dis.h"
#include "dis_.h"
#undef disrsc

signed char disrsc(stream, retval)
    int			stream;
    int			*retval;
    {
	int		locret;
	int		negate;
	signed char	value;
	unsigned	uvalue;

	assert(retval != NULL);
	assert(disr_commit != NULL);

	value = 0;
	switch (locret = disrsi_(stream, &negate, &uvalue, 1)) {
	    case DIS_SUCCESS:
		if (negate ? -uvalue >= SCHAR_MIN : uvalue <= SCHAR_MAX) {
			value = negate ? -uvalue : uvalue;
			break;
		} else
			locret = DIS_OVERFLOW;
	    case DIS_OVERFLOW:
		value = negate ? SCHAR_MIN : SCHAR_MAX;
	}
	*retval = ((*disr_commit)(stream, locret == DIS_SUCCESS) < 0) ?
	    DIS_NOCOMMIT : locret;
	return (value);
}
